/**
 * Clears specific cells in a given row on the sheet.
 * 
 * @param {GoogleAppsScript.Spreadsheet.Sheet} sheet - The sheet where the row will be cleared.
 * @param {number} row - The row index (1-based) where the cells will be cleared.
 */
function clearRow(sheet, row) {
  sheet.getRange(row+1, 1).setValue(false);
  sheet.getRange(row + 1, 2).clearContent(); // Clear column B
  sheet.getRange(row + 1, 4).clearContent(); // Clear column D
  sheet.getRange(row + 1, 5).clearContent(); // Clear column E
  sheet.getRange(row + 1, 6).clearContent(); // Clear column F
  sheet.getRange(row + 1, 7).clearContent(); // Clear column G
}

/**
 * Clears the working directory, as part of the daily trigger
 */
function clearWorkingAndDirectory() {
  let sheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working");
  const lastRow = sheet.getLastRow(); // Get the last row with data
  
  for (let i = 4; i < lastRow; i++) {  // i starts at 4, corresponding to row 5
    clearRow(sheet, i); // Clear specific cells in each row
  }

  sheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory");
  for(let i=2; i<sheet.getLastRow(); i++){
    if(sheet.getRange(i, 2).getValue()!='' && sheet.getRange(i,1).getValue()){
      sheet.getRange(i,1).setValue(false);
    }
  }
}

/**
 * Sets up a row by populating timestamp, last updated time, and user details.
 * Initializes the "check" column to true and ensures required fields are filled.
 * 
 * @param {GoogleAppsScript.Spreadsheet.Sheet} sheet - The sheet where the row will be set up.
 * @param {number} row - The row index (1-based) to set up.
 */
function setupRow(sheet, row) {
  const timestampCell = sheet.getRange(row, 5);
  const last_updated = sheet.getRange(row, 7);
  const name_cell = sheet.getRange(row, 2);
  const group_cell = sheet.getRange(row, 4);
  const check_cell = sheet.getRange(row, 1);

  check_cell.setValue(true); // Mark as checked

  const date = new Date();
  const formattedDate = Utilities.formatDate(date, "PST", "MM/dd/yyyy hh:mm a");
  timestampCell.setValue(formattedDate);

  if (last_updated.getValue().toString() === '') {
    last_updated.setValue(Utilities.formatDate(date, "PST", "hh:mm a"));
    last_updated.setNumberFormat("hh:mm AM/PM");
  }

  if (name_cell.getValue().toString() === '') {
    const userEmail = Session.getActiveUser().getEmail().toLowerCase().trim();
    const emailSheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Emails");
    const emailData = emailSheet.getRange("B2:E" + emailSheet.getLastRow()).getValues(); // B = email, E = handle

    let userHandle = null;
    for (let i = 0; i < emailData.length; i++) {
      if (emailData[i][0].toLowerCase().trim() === userEmail) {
        userHandle = emailData[i][4]; // Column E is index 4 (0-based)
        break;
        // return; 

      }
    }

    if (!userHandle) {
      name_cell.setValue("Enter Name");
      return;1
    }

    const directorySheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory");
    const directoryData = directorySheet.getRange("B2:D" + directorySheet.getLastRow()).getValues();

    for (let i = 0; i < directoryData.length; i++) {
      if (directoryData[i][0].toLowerCase().trim() === userHandle.toLowerCase().trim()) {
        const name = directoryData[i][1]; // Column C = Name
        const group = directoryData[i][2]; // Column D = Group
        name_cell.setValue(name || "Enter name");
        group_cell.setValue(group || "");
        return;
  }
}


    name_cell.setValue("Enter Name"); // Handle not in directory
  }
}

/**
 * Sets up a row in the sheet using a handle from the directory.
 * 
 * @param {GoogleAppsScript.Spreadsheet.Sheet} sheet - The sheet where the row will be set up.
 * @param {number} row - The row index (1-based) to set up.
 * @param {string} handle - The handle to populate the "name" cell.
 */
function setupRowFromDirectory(sheet, row, handle) {
  Logger.log("Setting up row");
  const timestampCell = sheet.getRange(row, 5);
  const last_updated = sheet.getRange(row, 7);
  const name_cell = sheet.getRange(row, 2);
  const check_cell = sheet.getRange(row, 1);

  check_cell.setValue(true); // Mark the row as active

  const date = new Date();
  const formattedDate = Utilities.formatDate(date, "PST", "MM/dd/yyyy hh:mm a");
  timestampCell.setValue(formattedDate);

  if (last_updated.getValue().toString() === '') {
    last_updated.setValue(Utilities.formatDate(date, "PST", "hh:mm a"));
    last_updated.setNumberFormat("hh:mm AM/PM");
  }

  if (name_cell.getValue().toString() === '') {
    name_cell.setValue(handle);
  }
}

/**
 * Sorts rows in the sheet by the most recent timestamp.
 * Rows with `true` in column A are sorted separately from others.
 * 
 * @param {GoogleAppsScript.Spreadsheet.Sheet} sheet - The sheet to sort.
 */
function sortMostRecent(sheet) {
  const sheetrange = sheet.getRange("A" + LIST_START_ROW + ":H");
  const data = sheetrange.getValues();

  // Separate rows based on the value in column A
  let trueRows = [];
  let otherRows = [];
  for (let i = 0; i < data.length; i++) {
    if (data[i][0] === true) {
      trueRows.push(data[i]);
    } else {
      otherRows.push(data[i]);
    }
  }

  // Sort both arrays by column 5 (index 4)
  trueRows.sort((a, b) => b[4] - a[4]);
  otherRows.sort((a, b) => b[4] - a[4]);

  // Combine the arrays
  const sortedData = trueRows.concat(otherRows);

  // Set the sorted data back to the sheet
  sheetrange.setValues(sortedData);

  // Reset formulas
  sheet.getRange("C" + LIST_START_ROW + ":C").setFormulas(sheet.getRange("J" + LIST_START_ROW + ":J").getFormulas());
}

/**
 * Checks for duplicate handles in an array.
 * 
 * @param {Array<string>} handles - An array of handle strings to check against.
 * @param {string} handle - The handle to check for duplicates.
 * @returns {boolean} True if no duplicate is found or the list is empty; false if a duplicate exists.
 */
function checkDuplicateLogic(handles, handle) {
  for (let h of handles) {
    if (h === "") return true; // No duplicate, end of list
    if (h === handle) return false; // Duplicate found
  }
  return true; // No duplicate found
}

/**
 * Gets the last row in column B, starting from row 5.
 * 
 * @param {GoogleAppsScript.Spreadsheet.Sheet} sheet - The sheet to search.
 * @returns {number} The index of the last non-empty row in column B.
 */
function getLastRowColumnB5(sheet) {
  const values = sheet.getRange("B5:B").getValues();
  for (let i = values.length - 1; i >= 0; i--) {
    if (values[i][0] !== "") {
      return i + 1;
    }
  }
  return 0;
}

/**
 * Finds the first empty row in the "DirectoryO" sheet starting from row 2.
 * 
 * @returns {number} The index of the first empty row.
 */
function emptyDirectoryRow2() {
  let i = 2;
  const directoryRef = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("DirectoryO");
  while (directoryRef.getRange(i, 2).getValue() !== "") {
    i++;
  }
  return i;
}

/**
 * Finds the row number of a handle in the directory, based on the current sheet.
 * 
 * @param {GoogleAppsScript.Spreadsheet.Sheet} sheet - The current sheet to use for lookup context.
 * @param {string} handle - The handle to search for in the directory.
 * @returns {number} The row number of the handle in the directory, or -1 if not found.
 */
function getNameRowInDirectory(sheet, handle){
  if(sheet.getName()==="Working"){
    let handleFound = false;
    let i=2;
    while(!handleFound){
      if(SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory").getRange(i, 2).getValue()===handle){
        handleFound = true;
        return i;
      }
      i++;
    }
  } else if(sheet.getName()==="Directory"){
    const sheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working");
    const lastRow = sheet.getLastRow();
    const handles = sheet.getRange("B2:B" + lastRow).getValues();

    for (let i = 0; i < handles.length; i++) {
      if (handles[i][0] === handle) {
        return i + 2; // Adding 2 because we start from row 2
      }
    }
  }
  return -1;
}

/**
 * Reminds the user upon check in that they need to close the lab door.
 */
function reminderToCloseDoor(){
  const ui = SpreadsheetApp.getUi();
  ui.alert("You're checked in! Reminder to close the lab door behind you!");
  
}

/**
 * At a time-based interval, unchecks and cleared rows where individuals are no longer checked in
 */
function uncheckAndClearExpiredRows() {
  const working = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working");
  const directory = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory");
  
  // Define the range where the helper column (J) exists
  var helperRange = working.getRange("J5:J100"); 
  var helperValues = helperRange.getValues(); 

  // Define the range for checkboxes in column A
  var checkboxesRange = working.getRange("A5:A100");
  var checkboxes = checkboxesRange.getValues();

  // Loop through each row to check expiration status
  for (var i = 0; i < helperValues.length; i++) {
    if (helperValues[i][0] === true || helperValues[i][0] == "TRUE") { // If Column J is TRUE (expired)
      checkboxes[i][0] = false; // Uncheck the checkbox in Column A
      
      // Clear Columns B through G for the expired row
      working.getRange(i + 5, 2, 1, 6).clearContent();
    }
  }

  var directoryHelperRange = directory.getRange("G2:G100");
  var directoryHelperValues = directoryHelperRange.getValues();
  var dirCheckboxesRange = directory.getRange("A2:A100");
  var dirCheckboxes = dirCheckboxesRange.getValues();

  for (var i = 0; i < directoryHelperValues.length; i++) {
    if(directoryHelperValues[i][0] == "") {
        //dirCheckboxes[i][0] = false;
        directory.getRange(i+2, 1).setValue(false);
    }
  }

  
  
  // Update checkboxes in one batch operation
  checkboxesRange.setValues(checkboxes);
  //dirCheckboxesRange.setValues(dirCheckboxes);
}








