/**
 * Sorts timestamps in the "Working" and "215L" sheets of the active spreadsheet.
 * Uses the `sortMostRecent` function to ensure the data is sorted by the most recent timestamps.
 */
function sortTimestamps() {
  sortMostRecent(SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working"));
  sortMostRecent(SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Directory"));
}

/**
 * Sets up a time-driven trigger to run the `checkTimestamps` function every 5 minutes.
 * This ensures timestamp checks are performed periodically without manual intervention.
 */
function createTimeDrivenTrigger() {
  ScriptApp.newTrigger('checkTimestamps')
    .timeBased()
    .everyMinutes(5)
    .create();
}

/**
 * Checks timestamps in the "Working" sheet and processes them.
 * Abstracts the logic for timestamp checks to simplify periodic execution.
 */
function checkTimestamps() {
  const sheet = SpreadsheetApp.getActiveSpreadsheet().getSheetByName("Working");
  checkTimestampsLocation(sheet);
}

/**
 * Handles edit events in the spreadsheet, routing them to the appropriate function based on the sheet name.
 * This allows different sheets to have custom behavior for edits.
 */
function onEditVerified(e) {
  const sheet = e.source.getActiveSheet(); // Active sheet for the edit event
  const sheetName = sheet.getName();
  const value = e.value;
  const range = e.range;

  if (sheetName === 'Working') {
    onlineTimestamp(sheet, range, value);
  } else if (sheetName === 'Directory') {
    directoryTimestamp(sheet, range, value);
  } else {
    // Add additional logic for other sheets here if needed
  }
}

/**
 * Sets up an edit-driven trigger for the `onEditVerified` function.
 * This trigger applies to the active spreadsheet and handles all edit events.
 */
function createEditDrivenTrigger() {
  ScriptApp.newTrigger('onEditVerified')
    .forSpreadsheet(SpreadsheetApp.getActiveSpreadsheet())
    .onEdit()
    .create();
}

/**
 * Creates a daily time-driven trigger to run the `getEmailsAndNames` function and updated directory based on it.
 * The trigger is configured to execute at 4 a.m. each day.
 */
function setDailyTrigger() {
  ScriptApp.newTrigger('getEmailsAndNames')
    .timeBased()
    .everyDays(1)
    .atHour(4)
    .create();
  ScriptApp.newTrigger("dailyUpdateDirectory")
    .timeBased()
    .everyDays(1)
    .atHour(4) 
    .create();

}

/**
 * Creates a dailty time-driven trigger to run the clearWorkingAndDirectory function and clear the Working tab at 2 AM every day.
 */
function createClearanceTrigger(){
  ScriptApp.newTrigger('clearWorkingAndDirectory')
    .timeBased()
    .everyDays(1)
    .atHour(2)
    .create();
}

/**
 * Temporarily deletes contents of H2 and immediately repopulates with discord and zoom link, as to not lose it
 */
function updateH2() {
  var sheet = SpreadsheetApp.getActiveSpreadsheet().getActiveSheet();
  var cell = sheet.getRange("H2");

  cell.clearContent();

  cell.setValue('Discord | Zoom');
  cell.setRichTextValue(
    SpreadsheetApp.newRichTextValue()
      .setText("Discord | Zoom")
      .setLinkUrl(0, 7, "https://discord.com/invite/8B5BYXPJPg") // 
      .setLinkUrl(10, 14, "https://washington.zoom.us/j/93329327422") 
      .build()
  );
}

/**
 * Creates the time based trigger for H2
 */
function createTimeTriggerForH2() {
  ScriptApp.newTrigger("updateH2")
    .timeBased()
    .everyHours(1)
    .create();
}

/**
 * Clears rows after their status has turned red
 */
function uncheckAndClearExpiredRowsTrigger(){
  ScriptApp.newTrigger("uncheckAndClearExpiredRows")
  .timeBased()
  .everyMinutes(1)
  .create();
}


// Legacy or unused functions (commented out for reference):
// function createTimeDrivenTrigger() {
//   ScriptApp.newTrigger('sortTimestamps')
//     .timeBased()
//     .everyMinutes(15)
//     .create();
// }

// function setSortTimestampsTrigger() {
//   ScriptApp.newTrigger('sortTimestamps')
//     .timeBased()
//     .everyMinutes(15)
//     .create();
// }
