/**
 * Runs all daily automation tasks:
 * 1. Updates "Directory" from "Emails"
 * 2. Sorts "Directory" by rank (descending)
 * 3. Syncs check-in statuses
 * 4. Handles inactivity and logouts
 * 5. Clears outdated data
 */
function dailyUpdateDirectory() {
  updateDirectoryFromEmails();
  sortDirectoryByRank();
  // checkDirectoryAndWorkingSheetsMatch();
  clearWorkingAndDirectory();
}

/**
 * Updates "Directory" from "Emails" and ensures all users are present.
 * Aligns names in Column B, ranks in Column C, and preserves comments in Column F.
 */
function updateDirectoryFromEmails() {
  const ss = SpreadsheetApp.getActiveSpreadsheet();
  const emailsSheet = ss.getSheetByName("Emails");
  const directorySheet = ss.getSheetByName("Directory");

  if (!emailsSheet || !directorySheet) {
    Logger.log("Error: 'Emails' or 'Directory' sheet is missing.");
    return;
  }

  // Get Names (Column E), Ranks (Column D), and existing Comments (Column F) from "Directory"
  const emailsData = emailsSheet.getRange("E2:E" + emailsSheet.getLastRow()).getValues();
  const ranksData = emailsSheet.getRange("D2:D" + emailsSheet.getLastRow()).getValues();
  const directoryData = directorySheet.getRange("B2:F" + directorySheet.getLastRow()).getValues();
  
  let directoryMap = new Map();
  
  // Load existing Directory data into a Map for fast lookups (Name -> Comment)
  directoryData.forEach(row => {
    if (row[0]) directoryMap.set(row[0], row[4]); // {Name -> Comment}
  });

  let updatedRows = [];

  for (let i = 0; i < emailsData.length; i++) {
    let name = emailsData[i][0];
    let rank = ranksData[i][0] || ""; // Ensure rank is recorded as empty if missing
    let comment = directoryMap.has(name) ? directoryMap.get(name) : ""; // Preserve existing comment
    
    if (name) { // Ensure only names with values are added
      updatedRows.push([name, rank, comment]); // Add or update user
    }
  }

  if (updatedRows.length > 0) {
    directorySheet.getRange(2, 2, updatedRows.length, 2).setValues(updatedRows.map(row => [row[0], row[1]]));
    directorySheet.getRange(2, 6, updatedRows.length, 1).setValues(updatedRows.map(row => [row[2]]));
  }
  
  Logger.log("Directory updated from Emails with preserved comments.");
}

/**
 * Sorts "Directory" by rank magnitude (Descending) while preserving comments.
 */
function sortDirectoryByRank() {
  const ss = SpreadsheetApp.getActiveSpreadsheet();
  const directorySheet = ss.getSheetByName("Directory");

  if (!directorySheet) {
    Logger.log("Error: 'Directory' sheet is missing.");
    return;
  }

  const lastRow = directorySheet.getLastRow();
  if (lastRow < 2) return; // No data to sort

  let data = directorySheet.getRange("B2:C" + lastRow).getValues();
  let comments = directorySheet.getRange("F2:F" + lastRow).getValues();

  // Merge comments with data before sorting
  let mergedData = data.map((row, index) => [...row, comments[index][0]]);

  // Sort by Rank (Column C, index 1) in descending order, keeping empty ranks at the bottom
  mergedData.sort((a, b) => {
    const rankA = parseInt(a[1]) || 0; // Convert rank to number, empty -> 0
    const rankB = parseInt(b[1]) || 0;
    return rankB - rankA; // Sort descending
  });

  // Write sorted data back to the sheet while keeping comments intact
  directorySheet.getRange(2, 2, mergedData.length, 2).setValues(mergedData.map(row => [row[0], row[1]]));
  directorySheet.getRange(2, 6, mergedData.length, 1).setValues(mergedData.map(row => [row[2]]));
  
  Logger.log("Directory sorted by rank with preserved comments.");
}

